﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using NDesk.Options;

namespace SendMessage
{
  class Program
  {

    #region Imports

    private const int HWND_BROADCAST = 0xffff;

    [DllImport("user32.dll")]
    private static extern IntPtr FindWindow(
      string lpClassName,  // window class name
      string lpWindowName  // window caption
      );

    [DllImport("user32.dll")]
    private static extern IntPtr SendMessage(
      IntPtr hWnd,   // handle to destination window
      uint Msg,      // message
      uint wParam,   // first message parameter
      uint lParam    // second message parameter
      );

    [DllImport("user32.dll")]
    private static extern IntPtr PostMessage(
      IntPtr hWnd,   // handle to destination window
      uint Msg,      // message
      uint wParam,   // first message parameter
      uint lParam    // second message parameter
      );

    #endregion Imports

    static void Main(string[] args)
    {
      bool show_help = false;
      string windowClass = null;
      string windowName = "";
      string processName = "";
      string message = "";
      string wParam = "";
      string lParam = "";
      string delay = "";

      OptionSet p = new OptionSet ()
        .Add("c|C=", "The window class name of the window to receive the message, optional.", v => windowClass = v)
        .Add("n|N=", "The window name, optional (choose either -n or -p).", v => windowName = v)
        .Add("p|P=", "The process name; sends the message to the first matching process, optional (choose either -n or -p).", v => processName = v)
        .Add("m|M=", "The windows message number to send (decimal), required.", v => message = v)
        .Add("w|W=", "The messages wParam value (decimal), required.", v => wParam = v)
        .Add("l|L=", "The messages lParam value (decimal), required.", v => lParam = v)
        .Add("d|D=", "Milliseconds to delay between each message sent, not yet implemented.", v => delay = v)
        .Add("h|help", "Show this message and exit.", v => show_help = v != null );

      List<string> extra;
      try
      {
        extra = p.Parse(args);

        // Check for required parameters.
        if (message.Length == 0 ||
          wParam.Length == 0 ||
          lParam.Length == 0)
        {
          show_help = true;
        }
      }
      catch (OptionException e)
      {
        Console.Write("SendMessage: ");
        Console.WriteLine(e.Message);
        Console.WriteLine("Try `SendMessage --help' for more information.");
        return;
      }

      if (show_help)
      {
        ShowHelp(p);
        return;
      }

      if (windowName.Length > 0)
      {
        IntPtr handle = FindWindow(windowClass, windowName);
        if (handle != IntPtr.Zero)
        {
          SendMessage(
            handle,
            Convert.ToUInt32(message),
            Convert.ToUInt32(wParam),
            Convert.ToUInt32(lParam));
        }
      }
      else if (processName.Length > 0)
      {
        Process[] processes = Process.GetProcessesByName(processName);
        SendMessage(
          (IntPtr)processes[0].MainWindowHandle,
          Convert.ToUInt32(message),
          Convert.ToUInt32(wParam),
          Convert.ToUInt32(lParam));
      }
    }

    static void ShowHelp(OptionSet p)
    {
      Console.WriteLine("Send one or more messages to any window(s).");
      Console.WriteLine();
      Console.WriteLine("Usage: SendMessage [OPTIONS]");
      Console.WriteLine();
      Console.WriteLine("Options:");
      p.WriteOptionDescriptions(Console.Out);
    }
  }
}
